@@ -4,12 +4,7 @@ import os
 from django.conf import settings
 from django.db import models
 from django.utils.encoding import python_2_unicode_compatible
-
-# Prior to Django 1.5, the AUTH_USER_MODEL setting does not exist.
-# Note that we don't perform this code in the compat module due to
-# bug report #1297
-# See: https://github.com/tomchristie/django-rest-framework/issues/1297
-AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')
+from django.utils.translation import ugettext_lazy as _
 
 
 @python_2_unicode_compatible
@@ -17,17 +12,22 @@ class Token(models.Model):
     """
     The default authorization token model.
     """
-    key = models.CharField(max_length=40, primary_key=True)
-    user = models.OneToOneField(AUTH_USER_MODEL, related_name='auth_token')
-    created = models.DateTimeField(auto_now_add=True)
+    key = models.CharField(_("Key"), max_length=40, primary_key=True)
+    user = models.OneToOneField(
+        settings.AUTH_USER_MODEL, related_name='auth_token',
+        on_delete=models.CASCADE, verbose_name=_("User")
+    )
+    created = models.DateTimeField(_("Created"), auto_now_add=True)
 
     class Meta:
         # Work around for a bug in Django:
         # https://code.djangoproject.com/ticket/19422
         #
         # Also see corresponding ticket:
-        # https://github.com/tomchristie/django-rest-framework/issues/705
+        # https://github.com/encode/django-rest-framework/issues/705
         abstract = 'rest_framework.authtoken' not in settings.INSTALLED_APPS
+        verbose_name = _("Token")
+        verbose_name_plural = _("Tokens")
 
     def save(self, *args, **kwargs):
         if not self.key:
