@@ -1,7 +1,7 @@
+from django.contrib.auth import authenticate
 from django.utils.translation import ugettext_lazy as _
 
 from rest_framework import serializers
-from rest_framework.compat import authenticate
 
 
 class AuthTokenSerializer(serializers.Serializer):
