@@ -7,6 +7,7 @@ See schemas.__init__.py for package overview.
 import re
 import warnings
 from collections import OrderedDict
+from weakref import WeakKeyDictionary
 
 from django.db import models
 from django.utils.encoding import force_text, smart_text
@@ -95,6 +96,8 @@ def field_to_schema(field):
             description=description,
             format='date-time'
         )
+    elif isinstance(field, serializers.JSONField):
+        return coreschema.Object(title=title, description=description)
 
     if field.style.get('base_template') == 'textarea.html':
         return coreschema.String(
@@ -102,6 +105,7 @@ def field_to_schema(field):
             description=description,
             format='textarea'
         )
+
     return coreschema.String(title=title, description=description)
 
 
@@ -125,6 +129,10 @@ class ViewInspector(object):
 
     Provide subclass for per-view schema generation
     """
+
+    def __init__(self):
+        self.instance_schemas = WeakKeyDictionary()
+
     def __get__(self, instance, owner):
         """
         Enables `ViewInspector` as a Python _Descriptor_.
@@ -141,9 +149,17 @@ class ViewInspector(object):
         See: https://docs.python.org/3/howto/descriptor.html for info on
         descriptor usage.
         """
+        if instance in self.instance_schemas:
+            return self.instance_schemas[instance]
+
         self.view = instance
         return self
 
+    def __set__(self, instance, other):
+        self.instance_schemas[instance] = other
+        if other is not None:
+            other.view = instance
+
     @property
     def view(self):
         """View property."""
@@ -186,6 +202,7 @@ class AutoSchema(ViewInspector):
         * `manual_fields`: list of `coreapi.Field` instances that
             will be added to auto-generated fields, overwriting on `Field.name`
         """
+        super(AutoSchema, self).__init__()
         if manual_fields is None:
             manual_fields = []
         self._manual_fields = manual_fields
@@ -230,9 +247,11 @@ class AutoSchema(ViewInspector):
         method_docstring = getattr(view, method_name, None).__doc__
         if method_docstring:
             # An explicit docstring on the method or action.
-            return formatting.dedent(smart_text(method_docstring))
+            return self._get_description_section(view, method.lower(), formatting.dedent(smart_text(method_docstring)))
+        else:
+            return self._get_description_section(view, getattr(view, 'action', method.lower()), view.get_view_description())
 
-        description = view.get_view_description()
+    def _get_description_section(self, view, header, description):
         lines = [line for line in description.splitlines()]
         current_section = ''
         sections = {'': ''}
@@ -246,7 +265,6 @@ class AutoSchema(ViewInspector):
 
         # TODO: SCHEMA_COERCE_METHOD_NAMES appears here and in `SchemaGenerator.get_keys`
         coerce_method_names = api_settings.SCHEMA_COERCE_METHOD_NAMES
-        header = getattr(view, 'action', method.lower())
         if header in sections:
             return sections[header].strip()
         if header in coerce_method_names:
@@ -452,6 +470,7 @@ class ManualSchema(ViewInspector):
         * `fields`: list of `coreapi.Field` instances.
         * `descripton`: String description for view. Optional.
         """
+        super(ManualSchema, self).__init__()
         assert all(isinstance(f, coreapi.Field) for f in fields), "`fields` must be a list of coreapi.Field instances"
         self._fields = fields
         self._description = description
@@ -471,9 +490,13 @@ class ManualSchema(ViewInspector):
         )
 
 
-class DefaultSchema(object):
+class DefaultSchema(ViewInspector):
     """Allows overriding AutoSchema using DEFAULT_SCHEMA_CLASS setting"""
     def __get__(self, instance, owner):
+        result = super(DefaultSchema, self).__get__(instance, owner)
+        if not isinstance(result, DefaultSchema):
+            return result
+
         inspector_class = api_settings.DEFAULT_SCHEMA_CLASS
         assert issubclass(inspector_class, ViewInspector), "DEFAULT_SCHEMA_CLASS must be set to a ViewInspector (usually an AutoSchema) subclass"
         inspector = inspector_class()
