@@ -85,6 +85,22 @@ class TestViewNamesAndDescriptions(TestCase):
             pass
         assert MockView().get_view_name() == 'Mock'
 
+    def test_view_name_uses_name_attribute(self):
+        class MockView(APIView):
+            name = 'Foo'
+        assert MockView().get_view_name() == 'Foo'
+
+    def test_view_name_uses_suffix_attribute(self):
+        class MockView(APIView):
+            suffix = 'List'
+        assert MockView().get_view_name() == 'Mock List'
+
+    def test_view_name_preferences_name_over_suffix(self):
+        class MockView(APIView):
+            name = 'Foo'
+            suffix = 'List'
+        assert MockView().get_view_name() == 'Foo'
+
     def test_view_description_uses_docstring(self):
         """Ensure view descriptions are based on the docstring."""
         class MockView(APIView):
@@ -112,6 +128,17 @@ class TestViewNamesAndDescriptions(TestCase):
 
         assert MockView().get_view_description() == DESCRIPTION
 
+    def test_view_description_uses_description_attribute(self):
+        class MockView(APIView):
+            description = 'Foo'
+        assert MockView().get_view_description() == 'Foo'
+
+    def test_view_description_allows_empty_description(self):
+        class MockView(APIView):
+            """Description."""
+            description = ''
+        assert MockView().get_view_description() == ''
+
     def test_view_description_can_be_empty(self):
         """
         Ensure that if a view has no docstring,
