@@ -7,16 +7,18 @@ an appropriate set of serializer fields for each case.
 """
 from __future__ import unicode_literals
 
+import datetime
 import decimal
+import sys
 from collections import OrderedDict
 
+import django
 import pytest
 from django.core.exceptions import ImproperlyConfigured
 from django.core.validators import (
     MaxValueValidator, MinLengthValidator, MinValueValidator
 )
 from django.db import models
-from django.db.models import DurationField as ModelDurationField
 from django.test import TestCase
 from django.utils import six
 
@@ -219,6 +221,25 @@ class TestRegularFieldMappings(TestCase):
             )
         self.assertEqual(unicode_repr(TestSerializer()), expected)
 
+    # merge this into test_regular_fields / RegularFieldsModel when
+    # Django 2.1 is the minimum supported version
+    @pytest.mark.skipif(django.VERSION < (2, 1), reason='Django version < 2.1')
+    def test_nullable_boolean_field(self):
+        class NullableBooleanModel(models.Model):
+            field = models.BooleanField(null=True, default=False)
+
+        class NullableBooleanSerializer(serializers.ModelSerializer):
+            class Meta:
+                model = NullableBooleanModel
+                fields = ['field']
+
+        expected = dedent("""
+            NullableBooleanSerializer():
+                field = BooleanField(allow_null=True, required=False)
+        """)
+
+        self.assertEqual(unicode_repr(NullableBooleanSerializer()), expected)
+
     def test_method_field(self):
         """
         Properties and methods on the model should be allowed as `Meta.fields`
@@ -349,7 +370,7 @@ class TestDurationFieldMapping(TestCase):
             """
             A model that defines DurationField.
             """
-            duration_field = ModelDurationField()
+            duration_field = models.DurationField()
 
         class TestSerializer(serializers.ModelSerializer):
             class Meta:
@@ -363,6 +384,31 @@ class TestDurationFieldMapping(TestCase):
         """)
         self.assertEqual(unicode_repr(TestSerializer()), expected)
 
+    def test_duration_field_with_validators(self):
+        class ValidatedDurationFieldModel(models.Model):
+            """
+            A model that defines DurationField with validators.
+            """
+            duration_field = models.DurationField(
+                validators=[MinValueValidator(datetime.timedelta(days=1)), MaxValueValidator(datetime.timedelta(days=3))]
+            )
+
+        class TestSerializer(serializers.ModelSerializer):
+            class Meta:
+                model = ValidatedDurationFieldModel
+                fields = '__all__'
+
+        expected = dedent("""
+            TestSerializer():
+                id = IntegerField(label='ID', read_only=True)
+                duration_field = DurationField(max_value=datetime.timedelta(3), min_value=datetime.timedelta(1))
+        """) if sys.version_info < (3, 7) else dedent("""
+            TestSerializer():
+                id = IntegerField(label='ID', read_only=True)
+                duration_field = DurationField(max_value=datetime.timedelta(days=3), min_value=datetime.timedelta(days=1))
+        """)
+        self.assertEqual(unicode_repr(TestSerializer()), expected)
+
 
 class TestGenericIPAddressFieldValidation(TestCase):
     def test_ip_address_validation(self):
@@ -381,7 +427,7 @@ class TestGenericIPAddressFieldValidation(TestCase):
                          '{0}'.format(s.errors))
 
 
-@pytest.mark.skipUnless(postgres_fields, 'postgres is required')
+@pytest.mark.skipif('not postgres_fields')
 class TestPosgresFieldsMapping(TestCase):
     def test_hstore_field(self):
         class HStoreFieldModel(models.Model):
@@ -1203,3 +1249,28 @@ class TestFieldSource(TestCase):
         """)
         self.maxDiff = None
         self.assertEqual(unicode_repr(TestSerializer()), expected)
+
+
+class Issue6110TestModel(models.Model):
+    """Model without .objects manager."""
+
+    name = models.CharField(max_length=64)
+    all_objects = models.Manager()
+
+
+class Issue6110ModelSerializer(serializers.ModelSerializer):
+    class Meta:
+        model = Issue6110TestModel
+        fields = ('name',)
+
+
+class Issue6110Test(TestCase):
+
+    def test_model_serializer_custom_manager(self):
+        instance = Issue6110ModelSerializer().create({'name': 'test_name'})
+        self.assertEqual(instance.name, 'test_name')
+
+    def test_model_serializer_custom_manager_error_message(self):
+        msginitial = ('Got a `TypeError` when calling `Issue6110TestModel.all_objects.create()`.')
+        with self.assertRaisesMessage(TypeError, msginitial):
+            Issue6110ModelSerializer().create({'wrong_param': 'wrong_param'})
