@@ -3,7 +3,7 @@ import uuid
 import pytest
 from _pytest.monkeypatch import MonkeyPatch
 from django.conf.urls import url
-from django.core.exceptions import ImproperlyConfigured
+from django.core.exceptions import ImproperlyConfigured, ObjectDoesNotExist
 from django.test import override_settings
 from django.utils.datastructures import MultiValueDict
 
@@ -167,6 +167,22 @@ class TestHyperlinkedRelatedField(APISimpleTestCase):
         representation = self.field.to_representation(MockObject(pk=''))
         assert representation is None
 
+    def test_serialize_empty_relationship_attribute(self):
+        class TestSerializer(serializers.Serializer):
+            via_unreachable = serializers.HyperlinkedRelatedField(
+                source='does_not_exist.unreachable',
+                view_name='example',
+                read_only=True,
+            )
+
+        class TestSerializable:
+            @property
+            def does_not_exist(self):
+                raise ObjectDoesNotExist
+
+        serializer = TestSerializer(TestSerializable())
+        assert serializer.data == {'via_unreachable': None}
+
     def test_hyperlinked_related_lookup_exists(self):
         instance = self.field.to_internal_value('http://example.org/example/foobar/')
         assert instance is self.queryset.items[0]
@@ -181,6 +197,36 @@ class TestHyperlinkedRelatedField(APISimpleTestCase):
         msg = excinfo.value.detail[0]
         assert msg == 'Invalid hyperlink - Object does not exist.'
 
+    def test_hyperlinked_related_internal_type_error(self):
+        class Field(serializers.HyperlinkedRelatedField):
+            def get_object(self, incorrect, signature):
+                raise NotImplementedError()
+
+        field = Field(view_name='example', queryset=self.queryset)
+        with pytest.raises(TypeError):
+            field.to_internal_value('http://example.org/example/doesnotexist/')
+
+    def hyperlinked_related_queryset_error(self, exc_type):
+        class QuerySet:
+            def get(self, *args, **kwargs):
+                raise exc_type
+
+        field = serializers.HyperlinkedRelatedField(
+            view_name='example',
+            lookup_field='name',
+            queryset=QuerySet(),
+        )
+        with pytest.raises(serializers.ValidationError) as excinfo:
+            field.to_internal_value('http://example.org/example/doesnotexist/')
+        msg = excinfo.value.detail[0]
+        assert msg == 'Invalid hyperlink - Object does not exist.'
+
+    def test_hyperlinked_related_queryset_type_error(self):
+        self.hyperlinked_related_queryset_error(TypeError)
+
+    def test_hyperlinked_related_queryset_value_error(self):
+        self.hyperlinked_related_queryset_error(ValueError)
+
 
 class TestHyperlinkedIdentityField(APISimpleTestCase):
     def setUp(self):
